/******************************************************************************/
/*  fichero: ccompiler.l                                                      */
/*  autores: Antonio Carrasco Grande,                                         */
/*           Rafael Gomez Lago      &                                         */
/*           Jesus Lopez Mora                                                 */
/*  fecha: enero 1,992.                                                       */
/*     Fichero fuente para el generador de scanners lex del analizador        */
/*  lexico del compilador de C--, propuesto como practica del curso de        */
/*  compiladores de quinto (curso academico 91/92)                            */
/******************************************************************************/

/* Definiciones */
{%
   USES LexLib;
}%

letra           [a-zA-Z]
digito          [0-9]
alfanumerico    [a-zA-Z_0-9]
codigo_escape	\\([abfnrtv"'?\\]|[0-7]{1,3}|x[0-9a-fA-F]+)
underscore      _
blanco          [ \t]
otros           .


%x	COMMENT

%%

"."    		                                return (PUNTO);
";"        	                                return (PUNTO_Y_COMA);
","    		                                return (COMA);
"&"        	                                return (AMPERSAND);
"("		                                return (P_AB);
")"		                                return (P_CE);
"{"		                                return (LL_AB);
"}"		                                return (LL_CE);
"->"		                                return (REF_UNION);
"="		                                return (ASIGNAC);
"+"		                                return (PLUS);
"-"		                                return (MENOS);
"++"		                                return (INC_UNARIO);
"--"		                                return (DEC_UNARIO);
"*"		                                return (ASTERISCO);
"/"		                                return (DIVIDE);
"%"		                                return (MODULO);
"+="		                                return (AUTOINC);
"-="		                                return (AUTODEC);
"*="		                                return (AUTOMUL);
"/="		                                return (AUTODIV);
"%="		                                return (AUTOMOD);
"=="		                                return (IGUAL_A);
"!="		                                return (DISTINTO);
">="		                                return (MAYOR_O_IGUAL);
"<="		                                return (MENOR_O_IGUAL);
">"		                                return (MAYOR_QUE);
"<"		                                return (MENOR_QUE);
"!"		                                return (NOT);
"&&"		                                return (AND);
"||"		                                return (OR);
{letra}{alfanumerico}*                          return (screen());
{digito}+		                        return (CONSTANTE_ENTERA);
\'([^'\\\n]|{esc})*\'                           return (CONSTANTE_CARACTER);
\"([^"\\\n]|{esc})*\"                           return (STRING);

{digito}+\.{digito}*((e|E)("+"|"-"){digito}+)?	return (FLOAT_CONSTANT);
\.{digito}+((e|E)("+"|"-"){digito}+)?		return (FLOAT_CONSTANT);

{blanco}+		;
\n			Inc (yylineno);

"/*"			BEGIN(COMMENT);

<COMMENT>"*/"		BEGIN(0);
<COMMENT>[^*\n]+	;		/* breaks comments into lines */
<COMMENT>\n		++yylineno;	/* so they won't overflow buffer */
<COMMENT>"*"		;		/* so they won't overflow buffer */

{otros}			return (yytext[1]);

%%

FUNCTION Es_Palabra_Clave (id : STRING; VAR token : INTEGER) : BOOLEAN;
  CONST logintud_identificador = 30;
  TYPE  Identificador = STRING [longitud_identificador];
  CONST
    (* tabla de palabras reservadas del C-- *)
    n_keywords = 7;  (*  Nmero de palabras claves.  *)
    keyword : ARRAY [1..n_keywords] OF Identificador = (
      'break', 'char', 'else', 'if', 'int', 'return', 'union');
    keyword_token : ARRAY [1..n_keywords] OF INTEGER = (
      BREAK, CHAR, ELSE, IF, INT, RETURN, UNION);
  VAR m, n, k : INTEGER;
  BEGIN
    m := 1; n := n_keywords;
    WHILE m<=n DO
      BEGIN
        k := m + (n - m) DIV 2;
        IF id = keyword [k] THEN
          BEGIN
            Es_Palabra_Clave := TRUE;
            token := keyword_token [k];
            EXIT
          END
        ELSE IF id > keyword [k]
               THEN m := k + 1
               ELSE n := k - 1
      END;
    Es_Palabra_Clave := FLASE
  END (*  Es_Palabra_Clave  *);
